/* vsgi-cookie-utils.c generated by valac 0.26.1, the Vala compiler
 * generated from vsgi-cookie-utils.vala, do not modify */

/*
 * This file is part of Valum.
 *
 * Valum is free software: you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * Valum is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Valum.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib.h>
#include <glib-object.h>
#include "vsgi.h"
#include <libsoup/soup.h>
#include <stdlib.h>
#include <string.h>

#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);



gboolean vsgi_str_const_equal (const gchar* a, const gchar* b);


/**
 * Sign the provided cookie name and value in-place using HMAC.
 *
 * The returned value will be 'HMAC(checksum_type, name + HMAC(checksum_type, value)) + value'
 * suitable for a cookie value which can the be verified with {@link VSGI.CookieUtils.verify}.
 *
 * {{{
 * CookieUtils.sign (cookie, ChecksumType.SHA512, "super-secret".data);
 * }}}
 *
 * @param cookie        cookie to sign
 * @param checksum_type hash algorithm used to compute the HMAC
 * @param key           secret used to sign the cookie name and value
 */
void vsgi_cookie_utils_sign (SoupCookie* cookie, GChecksumType checksum_type, guint8* key, int key_length1) {
	gchar* checksum = NULL;
	GChecksumType _tmp0_ = 0;
	guint8* _tmp1_ = NULL;
	gint _tmp1__length1 = 0;
	GChecksumType _tmp2_ = 0;
	guint8* _tmp3_ = NULL;
	gint _tmp3__length1 = 0;
	SoupCookie* _tmp4_ = NULL;
	const gchar* _tmp5_ = NULL;
	gchar* _tmp6_ = NULL;
	gchar* _tmp7_ = NULL;
	SoupCookie* _tmp8_ = NULL;
	const gchar* _tmp9_ = NULL;
	gchar* _tmp10_ = NULL;
	gchar* _tmp11_ = NULL;
	gchar* _tmp12_ = NULL;
	gchar* _tmp13_ = NULL;
	SoupCookie* _tmp14_ = NULL;
	SoupCookie* _tmp15_ = NULL;
	const gchar* _tmp16_ = NULL;
	gchar* _tmp17_ = NULL;
	gchar* _tmp18_ = NULL;
	g_return_if_fail (cookie != NULL);
	_tmp0_ = checksum_type;
	_tmp1_ = key;
	_tmp1__length1 = key_length1;
	_tmp2_ = checksum_type;
	_tmp3_ = key;
	_tmp3__length1 = key_length1;
	_tmp4_ = cookie;
	_tmp5_ = _tmp4_->value;
	_tmp6_ = g_compute_hmac_for_string (_tmp2_, _tmp3_, _tmp3__length1, _tmp5_, (gsize) (-1));
	_tmp7_ = _tmp6_;
	_tmp8_ = cookie;
	_tmp9_ = _tmp8_->name;
	_tmp10_ = g_strconcat (_tmp7_, _tmp9_, NULL);
	_tmp11_ = _tmp10_;
	_tmp12_ = g_compute_hmac_for_string (_tmp0_, _tmp1_, _tmp1__length1, _tmp11_, (gsize) (-1));
	_tmp13_ = _tmp12_;
	_g_free0 (_tmp11_);
	_g_free0 (_tmp7_);
	checksum = _tmp13_;
	_tmp14_ = cookie;
	_tmp15_ = cookie;
	_tmp16_ = _tmp15_->value;
	_tmp17_ = g_strconcat (checksum, _tmp16_, NULL);
	_tmp18_ = _tmp17_;
	soup_cookie_set_value (_tmp14_, _tmp18_);
	_g_free0 (_tmp18_);
	_g_free0 (checksum);
}


/**
 * Verify a signed cookie from {@link VSGI.CookieUtils.sign}.
 *
 * The signature is verified in constant time, more specifically a number
 * of comparisons equal to length of the checksum.
 *
 * @param cookie        cookie which signature will be verified
 * @param checksum_type hash algorithm used to compute the HMAC
 * @param key           secret used to sign the cookie's value
 * @param value         cookie's value extracted from its signature if the
 *                      verification succeeds, null otherwise
 * @return              true if the cookie is signed by the secret
 */
static glong string_strnlen (gchar* str, glong maxlen) {
	glong result = 0L;
	gchar* end = NULL;
	gchar* _tmp0_ = NULL;
	glong _tmp1_ = 0L;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	_tmp0_ = str;
	_tmp1_ = maxlen;
	_tmp2_ = memchr (_tmp0_, 0, (gsize) _tmp1_);
	end = _tmp2_;
	_tmp3_ = end;
	if (_tmp3_ == NULL) {
		glong _tmp4_ = 0L;
		_tmp4_ = maxlen;
		result = _tmp4_;
		return result;
	} else {
		gchar* _tmp5_ = NULL;
		gchar* _tmp6_ = NULL;
		_tmp5_ = end;
		_tmp6_ = str;
		result = (glong) (_tmp5_ - _tmp6_);
		return result;
	}
}


static gchar* string_substring (const gchar* self, glong offset, glong len) {
	gchar* result = NULL;
	glong string_length = 0L;
	gboolean _tmp0_ = FALSE;
	glong _tmp1_ = 0L;
	glong _tmp8_ = 0L;
	glong _tmp14_ = 0L;
	glong _tmp17_ = 0L;
	glong _tmp18_ = 0L;
	glong _tmp19_ = 0L;
	glong _tmp20_ = 0L;
	glong _tmp21_ = 0L;
	gchar* _tmp22_ = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp1_ = offset;
	if (_tmp1_ >= ((glong) 0)) {
		glong _tmp2_ = 0L;
		_tmp2_ = len;
		_tmp0_ = _tmp2_ >= ((glong) 0);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		glong _tmp3_ = 0L;
		glong _tmp4_ = 0L;
		glong _tmp5_ = 0L;
		_tmp3_ = offset;
		_tmp4_ = len;
		_tmp5_ = string_strnlen ((gchar*) self, _tmp3_ + _tmp4_);
		string_length = _tmp5_;
	} else {
		gint _tmp6_ = 0;
		gint _tmp7_ = 0;
		_tmp6_ = strlen (self);
		_tmp7_ = _tmp6_;
		string_length = (glong) _tmp7_;
	}
	_tmp8_ = offset;
	if (_tmp8_ < ((glong) 0)) {
		glong _tmp9_ = 0L;
		glong _tmp10_ = 0L;
		glong _tmp11_ = 0L;
		_tmp9_ = string_length;
		_tmp10_ = offset;
		offset = _tmp9_ + _tmp10_;
		_tmp11_ = offset;
		g_return_val_if_fail (_tmp11_ >= ((glong) 0), NULL);
	} else {
		glong _tmp12_ = 0L;
		glong _tmp13_ = 0L;
		_tmp12_ = offset;
		_tmp13_ = string_length;
		g_return_val_if_fail (_tmp12_ <= _tmp13_, NULL);
	}
	_tmp14_ = len;
	if (_tmp14_ < ((glong) 0)) {
		glong _tmp15_ = 0L;
		glong _tmp16_ = 0L;
		_tmp15_ = string_length;
		_tmp16_ = offset;
		len = _tmp15_ - _tmp16_;
	}
	_tmp17_ = offset;
	_tmp18_ = len;
	_tmp19_ = string_length;
	g_return_val_if_fail ((_tmp17_ + _tmp18_) <= _tmp19_, NULL);
	_tmp20_ = offset;
	_tmp21_ = len;
	_tmp22_ = g_strndup (((gchar*) self) + _tmp20_, (gsize) _tmp21_);
	result = _tmp22_;
	return result;
}


gboolean vsgi_cookie_utils_verify (SoupCookie* cookie, GChecksumType checksum_type, guint8* key, int key_length1, gchar** value) {
	gchar* _vala_value = NULL;
	gboolean result = FALSE;
	gint checksum_length = 0;
	GChecksumType _tmp0_ = 0;
	guint8* _tmp1_ = NULL;
	gint _tmp1__length1 = 0;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_ = NULL;
	gint _tmp4_ = 0;
	gint _tmp5_ = 0;
	gint _tmp6_ = 0;
	SoupCookie* _tmp7_ = NULL;
	const gchar* _tmp8_ = NULL;
	gint _tmp9_ = 0;
	gint _tmp10_ = 0;
	gint _tmp11_ = 0;
	gchar* checksum = NULL;
	GChecksumType _tmp12_ = 0;
	guint8* _tmp13_ = NULL;
	gint _tmp13__length1 = 0;
	GChecksumType _tmp14_ = 0;
	guint8* _tmp15_ = NULL;
	gint _tmp15__length1 = 0;
	SoupCookie* _tmp16_ = NULL;
	const gchar* _tmp17_ = NULL;
	gint _tmp18_ = 0;
	gchar* _tmp19_ = NULL;
	gchar* _tmp20_ = NULL;
	gchar* _tmp21_ = NULL;
	gchar* _tmp22_ = NULL;
	SoupCookie* _tmp23_ = NULL;
	const gchar* _tmp24_ = NULL;
	gchar* _tmp25_ = NULL;
	gchar* _tmp26_ = NULL;
	gchar* _tmp27_ = NULL;
	gchar* _tmp28_ = NULL;
	gint _tmp29_ = 0;
	const gchar* _tmp30_ = NULL;
	gint _tmp31_ = 0;
	gint _tmp32_ = 0;
	const gchar* _tmp33_ = NULL;
	SoupCookie* _tmp34_ = NULL;
	const gchar* _tmp35_ = NULL;
	gint _tmp36_ = 0;
	gchar* _tmp37_ = NULL;
	gchar* _tmp38_ = NULL;
	gboolean _tmp39_ = FALSE;
	gboolean _tmp40_ = FALSE;
	g_return_val_if_fail (cookie != NULL, FALSE);
	_tmp0_ = checksum_type;
	_tmp1_ = key;
	_tmp1__length1 = key_length1;
	_tmp2_ = g_compute_hmac_for_string (_tmp0_, _tmp1_, _tmp1__length1, "", (gsize) (-1));
	_tmp3_ = _tmp2_;
	_tmp4_ = strlen (_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = _tmp5_;
	_g_free0 (_tmp3_);
	checksum_length = _tmp6_;
	_g_free0 (_vala_value);
	_vala_value = NULL;
	_tmp7_ = cookie;
	_tmp8_ = _tmp7_->value;
	_tmp9_ = strlen (_tmp8_);
	_tmp10_ = _tmp9_;
	_tmp11_ = checksum_length;
	if (_tmp10_ < _tmp11_) {
		result = FALSE;
		if (value) {
			*value = _vala_value;
		} else {
			_g_free0 (_vala_value);
		}
		return result;
	}
	_tmp12_ = checksum_type;
	_tmp13_ = key;
	_tmp13__length1 = key_length1;
	_tmp14_ = checksum_type;
	_tmp15_ = key;
	_tmp15__length1 = key_length1;
	_tmp16_ = cookie;
	_tmp17_ = _tmp16_->value;
	_tmp18_ = checksum_length;
	_tmp19_ = string_substring (_tmp17_, (glong) _tmp18_, (glong) (-1));
	_tmp20_ = _tmp19_;
	_tmp21_ = g_compute_hmac_for_string (_tmp14_, _tmp15_, _tmp15__length1, _tmp20_, (gsize) (-1));
	_tmp22_ = _tmp21_;
	_tmp23_ = cookie;
	_tmp24_ = _tmp23_->name;
	_tmp25_ = g_strconcat (_tmp22_, _tmp24_, NULL);
	_tmp26_ = _tmp25_;
	_tmp27_ = g_compute_hmac_for_string (_tmp12_, _tmp13_, _tmp13__length1, _tmp26_, (gsize) (-1));
	_tmp28_ = _tmp27_;
	_g_free0 (_tmp26_);
	_g_free0 (_tmp22_);
	_g_free0 (_tmp20_);
	checksum = _tmp28_;
	_tmp29_ = checksum_length;
	_tmp30_ = checksum;
	_tmp31_ = strlen (_tmp30_);
	_tmp32_ = _tmp31_;
	_vala_assert (_tmp29_ == _tmp32_, "checksum_length == checksum.length");
	_tmp33_ = checksum;
	_tmp34_ = cookie;
	_tmp35_ = _tmp34_->value;
	_tmp36_ = checksum_length;
	_tmp37_ = string_substring (_tmp35_, (glong) 0, (glong) _tmp36_);
	_tmp38_ = _tmp37_;
	_tmp39_ = vsgi_str_const_equal (_tmp33_, _tmp38_);
	_tmp40_ = _tmp39_;
	_g_free0 (_tmp38_);
	if (_tmp40_) {
		SoupCookie* _tmp41_ = NULL;
		const gchar* _tmp42_ = NULL;
		gint _tmp43_ = 0;
		gchar* _tmp44_ = NULL;
		_tmp41_ = cookie;
		_tmp42_ = _tmp41_->value;
		_tmp43_ = checksum_length;
		_tmp44_ = string_substring (_tmp42_, (glong) _tmp43_, (glong) (-1));
		_g_free0 (_vala_value);
		_vala_value = _tmp44_;
		result = TRUE;
		_g_free0 (checksum);
		if (value) {
			*value = _vala_value;
		} else {
			_g_free0 (_vala_value);
		}
		return result;
	} else {
		result = FALSE;
		_g_free0 (checksum);
		if (value) {
			*value = _vala_value;
		} else {
			_g_free0 (_vala_value);
		}
		return result;
	}
	_g_free0 (checksum);
	if (value) {
		*value = _vala_value;
	} else {
		_g_free0 (_vala_value);
	}
}



